@@ -7,24 +7,24 @@ use integer;
 use B;
 use Carp ();
 use Time::Local qw/timelocal_nocheck timegm_nocheck/;
-use Encode qw/decode is_utf8/;
+use Encode qw/is_utf8 decode encode_utf8/;
 use Encode::Locale;
-use Locale::Scope qw/locale_scope/;
+use DateTime::Locale;
 use List::MoreUtils qw/uniq/;
 use POSIX qw/strftime LC_ALL/;
 use Time::Strptime::TimeZone;
+use DateTime::Format::Strptime;
 
 use constant DEBUG => exists $ENV{PERL_TIME_STRPTIME_DEBUG} && $ENV{PERL_TIME_STRPTIME_DEBUG};
 
-our $VERSION = 0.07;
+our $VERSION = 0.99;
 
 our %DEFAULT_HANDLER = (
-    '%' => [char          => '%' ],
     A   => [SKIP          => sub {
-        map quotemeta, map { ($_, substr $_, 0, 3) } map { is_utf8($_) ? $_ : decode(locale => $_) } map {
-            strftime('%a', 0, 0, 0, $_, 0, 0),
-            strftime('%A', 0, 0, 0, $_, 0, 0)
-        } 1..7
+        my $self = shift;
+        my $wide = $self->{locale}->day_format_wide;
+        my $abbr = $self->{locale}->day_format_abbreviated;
+        return [map quotemeta, map { lc, uc, $_ } map { is_utf8($_) ? $_ : decode(locale => $_) } map { $wide->[$_], $abbr->[$_] } 0..6];
     }],
     a   => [extend        => q{%A} ],
     B   => [localed_month => sub {
@@ -32,13 +32,17 @@ our %DEFAULT_HANDLER = (
 
         unless (exists $self->{format_table}{localed_month}) {
             my %format_table;
-            for my $month (1..12) {
-                my $b = strftime('%b', 0, 0, 0, 1, $month-1, 0);
-                my $B = strftime('%B', 0, 0, 0, 1, $month-1, 0);
-                $format_table{is_utf8($b) ? $b : decode(locale => $b) } = $month;
-                $format_table{is_utf8($B) ? $B : decode(locale => $B) } = $month;
+
+            my $wide = $self->{locale}->month_format_wide;
+            my $abbr = $self->{locale}->month_format_abbreviated;
+            for my $month (0..11) {
+                for my $key ($wide->[$month], $abbr->[$month]) {
+                    $key = decode(locale => $key) unless is_utf8 $key;
+                    $format_table{$key}    = $month + 1;
+                    $format_table{lc $key} = $month + 1;
+                    $format_table{uc $key} = $month + 1;
+                }
             }
-            $format_table{substr($_, 0, 3)} = $format_table{$_} for keys %format_table;
             $self->{format_table}{localed_month} = \%format_table;
         }
 
@@ -51,19 +55,28 @@ our %DEFAULT_HANDLER = (
     d   => [day         => ['0[1-9]','[12][0-9]','3[01]'] ],
     e   => [day         => [' [1-9]','[12][0-9]','3[01]'] ],
     F   => [extend      => q{%Y-%m-%d} ],
-    G   => [year        => q{%Y} ], ## It's realy OK?
-    g   => [SKIP        => q{[0-9]{2}} ],
+    G   => ['UNSUPPORTED'],
+    g   => ['UNSUPPORTED'],
     H   => [hour24      => ['[01][0-9]','2[0-3]'] ],
-    h   => [extend      => q{%b}                     ],
+    h   => [extend      => q{%B}                  ],
     I   => [hour12      => ['0[1-9]', '1[0-2]'] ],
-    j   => [day365      => ['00[1-9]','[12][0-9][0-9]','3[0-5][0-9]','36[0-6]'] ],
+    j   => [day365      => ['00[1-9]', '0[1-9][0-9]', '[12][0-9][0-9]','3[0-5][0-9]','36[0-6]'] ],
     k   => [hour24      => ['[ 1][0-9]','2[0-3]'] ],
     l   => [hour12      => [' [1-9]', '1[0-2]'] ],
     M   => [minute      => q{[0-5][0-9]}          ],
     m   => [month       => ['0[1-9]','1[0-2]']    ],
     n   => [SKIP        => q{\s+}                 ],
-    p   => [ampm        => q{[AP]M}],
-    P   => [ampm        => q{[ap]m}],
+    p   => [localed_pm  => sub {
+        my $self = shift;
+        unless (exists $self->{format_table}{localed_pm}) {
+            for my $pm (0, 1) {
+                my $key = $self->{locale}->am_pm_abbreviated->[$pm];
+                $key = decode(locale => $key) unless is_utf8 $key;
+                $self->{format_table}{localed_pm}{$key} = $pm;
+            }
+        }
+        return [map quotemeta, keys %{ $self->{format_table}{localed_pm} }];
+    }],
     R   => [extend      => q{%H:%M}               ],
     r   => [extend      => q{%I:%M:%S %p}         ],
     S   => [second      => ['[0-5][0-9]','60']    ],
@@ -97,7 +110,7 @@ sub new {
     my $self = bless +{
         format    => $format,
         time_zone => Time::Strptime::TimeZone->new($options->{time_zone}),
-        locale    => $options->{locale} || 'C',
+        locale    => DateTime::Locale->load($options->{locale} || 'C'),
         _handler  => +{
             %DEFAULT_HANDLER,
             %{ $options->{handler} || {} },
@@ -126,7 +139,6 @@ sub _compile_format {
 
     my $parser = do {
         # setlocale
-        my $locale    = locale_scope(LC_ALL, $self->{locale});
         my $time_zone = $self->{time_zone};
 
         # assemble format to regexp
@@ -173,7 +185,7 @@ EOD
 
         my $combined_src = sprintf $parser_src, B::perlstring(B::perlstring($self->{format})), $formatter_src;
         $self->{parser_src} = $combined_src;
-        warn Encode::encode_utf8 "[DEBUG] src: $combined_src" if DEBUG;
+        warn encode_utf8 "[DEBUG] src: $combined_src" if DEBUG;
 
         my $format_table = $self->{format_table} || {};
         eval $combined_src; ## no critic
@@ -205,8 +217,7 @@ sub _assemble_format {
         return $val;
     }
     else {
-        return ''             if $type eq 'TODO' and warn "SKIP(TODO): \%$c. patches welcome :)";
-        return quotemeta $val if $type eq 'SKIP';
+        return "(?:$val)"     if $type eq 'SKIP';
         return quotemeta $val if $type eq 'char';
 
         push @$types => $type;
@@ -235,36 +246,31 @@ EOD
 sub _gen_calc_epoch_src {
     my ($self, $types_table) = @_;
 
-    my $src = '';
-
     # hour24&minute&second
     # year&day365 or year&month&day
     my $second        = $types_table->{second} ? '$second' : 0;
     my $minute        = $types_table->{minute} ? '$minute' : 0;
     my $hour          = $self->_gen_calc_hour_src($types_table);
     if ($types_table->{epoch}) {
-        # nothing to do
+        return ''; # nothing to do
     }
     elsif ($types_table->{year} && $types_table->{month} && $types_table->{day}) {
-        $src .= <<EOD;
+        return <<EOD;
     \$epoch = timegm_nocheck($second, $minute, $hour, \$day, \$month - 1, \$year);
 EOD
     }
     elsif ($types_table->{year} && $types_table->{month}) {
-        $src .= <<EOD;
+        return <<EOD;
     \$epoch = timegm_nocheck($second, $minute, $hour, 1, \$month - 1, \$year);
 EOD
     }
     elsif ($types_table->{year} && $types_table->{day365}) {
-        $src .= <<EOD;
-    \$epoch = timegm_nocheck($second, $minute, $hour, 1, 0, \$year) + \$day365 * 60 * 60 * 24;
+        return <<EOD;
+    \$epoch = timegm_nocheck($second, $minute, $hour, 1, 0, \$year) + (\$day365 - 1) * 60 * 60 * 24;
 EOD
     }
-    else {
-        die 'unknown case. types: '. join ', ', keys %$types_table;
-    }
 
-    return $src;
+    die 'unknown case. types: '. join ', ', keys %$types_table;
 }
 
 sub _gen_calc_offset_src {
@@ -295,9 +301,11 @@ EOD
 EOD
     }
 
-    $src .= <<'EOD' unless defined $fixed_offset;
+    if (!defined $fixed_offset && !$types_table->{epoch}) {
+        $src .= <<'EOD'
     $epoch -= $offset;
 EOD
+    }
 
     return $src;
 }
@@ -308,13 +316,11 @@ sub _gen_calc_hour_src {
     if ($types_table->{hour24}) {
         return '$hour24';
     }
-    elsif ($types_table->{hour12} && $types_table->{ampm}) {
-        return <<'EOD';
-($hour12 == 12 ? (uc $ampm eq q{AM} ? 0 : 12) : ($hour12 + (uc $ampm eq q{PM} ? 12 : 0)))
-EOD
+    elsif ($types_table->{hour12} && $types_table->{pm}) {
+        return '(0,12)[$pm] + ($hour12 % 12)';
     }
     else {
-        return 0;
+        return '0';
     }
 }
 
@@ -329,9 +335,10 @@ sub _fixed_offset {
 1;
 __END__
 
-
 =encoding utf-8
 
+=for stopwords strptime
+
 =head1 NAME
 
 Time::Strptime::Format - L<strptime(3)> format compiler and parser.
@@ -346,10 +353,129 @@ Time::Strptime::Format - L<strptime(3)> format compiler and parser.
 
 =head1 DESCRIPTION
 
-B<THE SOFTWARE IS IT'S IN ALPHA QUALITY. IT MAY CHANGE THE API WITHOUT NOTICE.>
-
 This is L<Time::Strptime> engine.
 
+=head1 METHODS
+
+This class offers the following methods.
+
+=head2 Time::Strptime::Format->new($format, \%args)
+
+This methods creates a new format object. It accepts the following arguments:
+
+=over 4
+
+=item * time_zone
+
+The default time zone to use for objects returned from parsing.
+
+=item * locale
+
+The locale to use for objects returned from parsing.
+
+=back
+
+=head2 $strptime->parse($string)
+
+Given a string in the pattern specified in the constructor, this method will return the epoch and offset.
+If given a string that doesn't match the pattern, the formatter will throw the error.
+
+=head1 STRPTIME PATTERN TOKENS
+
+The following tokens are allowed in the pattern string for strptime:
+
+=over 4
+
+=item * %%
+
+The % character.
+
+=item * %a or %A
+
+The weekday name according to the current locale, in abbreviated form or the full name. (ignored)
+
+=item * %b or %B or %h
+
+The month name according to the current locale, in abbreviated form or the full name.
+
+=item * %d or %e
+
+The day of month (01-31). This will parse single digit numbers as well.
+
+=item * %D
+
+Equivalent to %m/%d/%y. (This is the American style date, very confusing to non-Americans, especially since %d/%m/%y is widely used in Europe. The ISO 8601 standard pattern is %F.)
+
+=item * %F
+
+Equivalent to %Y-%m-%d. (This is the ISO style date)
+
+=item * %H
+
+The hour (00-23). This will parse single digit numbers as well.
+
+=item * %I
+
+The hour on a 12-hour clock (1-12).
+
+=item * %j
+
+The day number in the year (1-366).
+
+=item * %m
+
+The month number (01-12). This will parse single digit numbers as well.
+
+=item * %M
+
+The minute (00-59). This will parse single digit numbers as well.
+
+=item * %n
+
+Arbitrary white-space. (ignored)
+
+=item * %p
+
+The equivalent of AM or PM according to the locale in use. (See L<DateTime::Locale>)
+
+=item * %r
+
+Equivalent to %I:%M:%S %p.
+
+=item * %R
+
+Equivalent to %H:%M.
+
+=item * %s
+
+Number of seconds since the Epoch.
+
+=item * %S
+
+The second (0-60; 60 may occur for leap seconds.).
+
+=item * %t
+
+Tab space. (ignored)
+
+=item * %T
+
+Equivalent to %H:%M:%S.
+
+=item * %Y
+
+A 4-digit year, including century (for example, 1991).
+
+=item * %z
+
+An RFC-822/ISO 8601 standard time zone specification. (e.g. +1100)
+
+=item * %Z
+
+The time zone name. (e.g. EST)
+
+=back
+
 =head1 LICENSE
 
 Copyright (C) karupanerura.
